import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import 'rxjs/add/operator/catch';
import { AuthenticationService } from './authentication.service';
import { ConfigurationService } from './configuration.service';
import { getError } from './authentication.service';
import { LoadingService } from './loading.service';
var APIService = (function () {
    function APIService(authentication, config, http, loading) {
        var _this = this;
        this.authentication = authentication;
        this.config = config;
        this.http = http;
        this.status = new BehaviorSubject({ loading: false });
        this.backendAvailable = new BehaviorSubject(true);
        loading.status.subscribe(function (isLoading) {
            _this.status.next({ loading: isLoading });
        });
    }
    APIService.prototype.get = function (path) {
        var url = this.getFullPath(path);
        var headers = this.authentication.getHeaders();
        return this.wrapRequest(this.http.get(url, { headers: headers }));
    };
    APIService.prototype.post = function (path, data) {
        var url = this.getFullPath(path);
        var headers = this.authentication.getHeaders();
        return this.wrapRequest(this.http.post(url, data, { headers: headers }));
    };
    APIService.prototype.patch = function (path, data) {
        var url = this.getFullPath(path);
        var headers = this.authentication.getHeaders();
        if (this.config.get('PATCH_RETURNS_REPRESENTATION', true)) {
            headers = headers.set('Prefer', 'return=representation');
        }
        return this.wrapRequest(this.http.patch(url, data, { headers: headers }));
    };
    APIService.prototype.delete = function (path) {
        var url = this.getFullPath(path);
        var headers = this.authentication.getHeaders();
        return this.wrapRequest(this.http.delete(url, { headers: headers }));
    };
    APIService.prototype.download = function (path) {
        var url = this.getFullPath(path);
        var headers = this.authentication.getHeaders();
        headers = headers.set('Content-Type', 'application/x-www-form-urlencoded');
        return this.wrapRequest(this.http.get(url, {
            responseType: 'blob',
            headers: headers
        }).map(function (blob) {
            return blob;
        }));
    };
    APIService.prototype.getFullPath = function (path) {
        var base = this.config.get('BACKEND_URL');
        // if path is already prefixed by base, no need to prefix twice
        // if path is already a full url no need to prefix either
        if (path.startsWith(base) || path.startsWith('http:') || path.startsWith('https:')) {
            return path;
        }
        else {
            return base + path;
        }
    };
    APIService.prototype.wrapRequest = function (request) {
        var _this = this;
        var timeout = this.config.get('CLIENT_TIMEOUT', 15000);
        var attempts = 0;
        return request
            .timeout(timeout)
            .retryWhen(function (errors) {
            /* retry when backend unavailable errors */
            return errors.delayWhen(function (response) {
                if ([0, 502, 503, 504].indexOf(response.status) >= 0) {
                    if (attempts < _this.config.get('RETRY_REQUEST_ATTEMPTS', 3)) {
                        attempts += 1;
                        return Observable.timer(_this.config.get('RETRY_REQUEST_DELAY', 2000));
                    }
                    _this.setBackendAvailability(false);
                }
                return Observable.throw(response);
            });
        })
            .do(function () { return _this.setBackendAvailability(true); })
            .catch(function (errorResponse) {
            return Observable.throw(getError(errorResponse));
        });
    };
    /* Emits only if it has changed */
    /* Emits only if it has changed */
    APIService.prototype.setBackendAvailability = /* Emits only if it has changed */
    function (availability) {
        if (this.backendAvailable.getValue() !== availability) {
            this.backendAvailable.next(availability);
        }
    };
    APIService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    APIService.ctorParameters = function () { return [
        { type: AuthenticationService, },
        { type: ConfigurationService, },
        { type: HttpClient, },
        { type: LoadingService, },
    ]; };
    return APIService;
}());
export { APIService };
