import { isPlatformBrowser } from '@angular/common';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Inject, Injectable, PLATFORM_ID } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { ConfigurationService } from './configuration.service';
var AuthenticationService = (function () {
    function AuthenticationService(config, http, platformId) {
        this.config = config;
        this.http = http;
        this.platformId = platformId;
        this.isAuthenticated = new BehaviorSubject({ state: false, username: null });
        if (isPlatformBrowser(this.platformId)) {
            var token = localStorage.getItem('auth');
            var lastLogin = localStorage.getItem('auth_time');
            // token expires after 12 hours
            var expire = config.get('AUTH_TOKEN_EXPIRES', 12 * 60 * 60 * 1000);
            if (!lastLogin || (Date.now() - Date.parse(lastLogin) > expire)) {
                localStorage.removeItem('auth');
                token = null;
            }
            if (token) {
                this.isAuthenticated.next({ state: true, username: this.getUsername() });
            }
        }
    }
    AuthenticationService.prototype.getUsername = function () {
        var userTokenInfo = this.getUserTokenInfo();
        if (userTokenInfo === null) {
            return null;
        }
        else {
            return userTokenInfo.username || userTokenInfo.sub || null;
        }
    };
    AuthenticationService.prototype.getUserTokenInfo = function () {
        if (isPlatformBrowser(this.platformId)) {
            var token = localStorage.getItem('auth');
            if (token) {
                var tokenParts = token.split('.');
                return JSON.parse(atob(tokenParts[1]));
            }
            else {
                return null;
            }
        }
        else {
            return null;
        }
    };
    AuthenticationService.prototype.login = function (login, password) {
        var _this = this;
        if (isPlatformBrowser(this.platformId)) {
            var headers = this.getHeaders();
            var body = JSON.stringify({
                login: login,
                password: password
            });
            return this.http.post(this.config.get('BACKEND_URL') + '/@login', body, { headers: headers })
                .do(function (data) {
                if (data.token) {
                    localStorage.setItem('auth', data['token']);
                    localStorage.setItem('auth_time', (new Date()).toISOString());
                    _this.isAuthenticated.next({ state: true, username: _this.getUsername() });
                }
                else {
                    localStorage.removeItem('auth');
                    localStorage.removeItem('auth_time');
                    _this.isAuthenticated.next({ state: false, username: null });
                }
            })
                .catch(function (errorResponse) {
                localStorage.removeItem('auth');
                localStorage.removeItem('auth_time');
                var error = getError(errorResponse);
                _this.isAuthenticated.next({ state: false, username: null, error: error.message });
                return Observable.throw(error);
            });
        }
        else {
            return Observable.of({});
        }
    };
    AuthenticationService.prototype.logout = function () {
        if (isPlatformBrowser(this.platformId)) {
            localStorage.removeItem('auth');
            localStorage.removeItem('auth_time');
            this.isAuthenticated.next({ state: false, username: null });
        }
    };
    AuthenticationService.prototype.requestPasswordReset = function (login) {
        var headers = this.getHeaders();
        var url = this.config.get('BACKEND_URL') + ("/@users/" + login + "/reset-password");
        return this.http.post(url, {}, { headers: headers })
            .catch(this.error.bind(this));
    };
    AuthenticationService.prototype.passwordReset = function (resetInfo) {
        var headers = this.getHeaders();
        var data = {
            new_password: resetInfo.newPassword
        };
        if (resetInfo.oldPassword) {
            data['old_password'] = resetInfo.oldPassword;
        }
        if (resetInfo.token) {
            data['reset_token'] = resetInfo.token;
        }
        var url = this.config.get('BACKEND_URL') + ("/@users/" + resetInfo.login + "/reset-password");
        return this.http.post(url, data, { headers: headers })
            .catch(this.error.bind(this));
    };
    AuthenticationService.prototype.getHeaders = function () {
        var headers = new HttpHeaders();
        headers = headers.set('Accept', 'application/json');
        headers = headers.set('Content-Type', 'application/json');
        if (isPlatformBrowser(this.platformId)) {
            var auth = localStorage.getItem('auth');
            if (auth) {
                headers = headers.set('Authorization', 'Bearer ' + auth);
            }
        }
        return headers;
    };
    AuthenticationService.prototype.error = function (errorResponse) {
        var error = getError(errorResponse);
        return Observable.throw(error);
    };
    AuthenticationService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    AuthenticationService.ctorParameters = function () { return [
        { type: ConfigurationService, },
        { type: HttpClient, },
        { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] },] },
    ]; };
    return AuthenticationService;
}());
export { AuthenticationService };
export function getError(errorResponse) {
    var error;
    if (errorResponse.error) {
        var errorResponseError = errorResponse.error;
        try {
            // string plone error
            errorResponseError = JSON.parse(errorResponseError);
            if (errorResponseError.error && errorResponseError.error.message) {
                // two levels of error properties
                error = errorResponseError.error;
            }
            else {
                error = errorResponseError;
            }
        }
        catch (SyntaxError) {
            if (errorResponseError.message && errorResponseError.type) {
                // object plone error
                error = errorResponseError;
            }
            else if (typeof errorResponseError.error === 'object' && errorResponseError.error.type) {
                // object plone error with two levels of error properties
                error = errorResponseError.error;
            }
            else {
                // not a plone error
                error = { type: errorResponse.statusText, message: errorResponse.message, traceback: [] };
            }
        }
    }
    else {
        error = { type: errorResponse.statusText, message: errorResponse.message, traceback: [] };
    }
    error.response = errorResponse;
    return error;
}
